@@ -16,7 +16,7 @@
 gSOAP XML Web services tools
 Copyright (C) 2001-2008, Robert van Engelen, Genivia, Inc. All Rights Reserved.
 This software is released under one of the following two licenses:
-GPL or Genivia's license for commercial use.
+GPL.
 --------------------------------------------------------------------------------
 GPL license.
 
@@ -57,20 +57,21 @@ int main()
 {
   struct soap soap;
   double a, b, result;
+  /* Init gSOAP context */
+  soap_init(&soap);
+  a = 10.0;
+  b = 20.0;
+#if defined(WITH_OPENSSL)
   /* Uncomment to call this first before all else if SSL is initialized elsewhere, e.g. in application code */
   /* soap_ssl_noinit(); */
   /* Init SSL before any threads are started (do this just once) */
   soap_ssl_init();
-  /* set up SSL locks */
+  /* set up SSL locks (not needed for OpenSSL 1.1.0 and greater) */
   if (CRYPTO_thread_setup())
   {
     fprintf(stderr, "Cannot setup thread mutex for OpenSSL\n");
     exit(1);
   }
-  a = 10.0;
-  b = 20.0;
-  /* Init gSOAP context */
-  soap_init(&soap);
   /* The supplied server certificate "server.pem" assumes that the server is
      running on 'localhost', so clients can only connect from the same host when
      verifying the server's certificate. Use SOAP_SSL_NO_AUTHENTICATION to omit
@@ -129,6 +130,7 @@ int main()
     soap_print_fault(&soap, stderr);
     exit(EXIT_FAILURE);
   }
+#endif
   soap.connect_timeout = 30;	/* try to connect for up to 30 seconds */
   soap.send_timeout = soap.recv_timeout = 10;	/* max I/O idle time is 10 seconds */
   if (soap_call_ns__add(&soap, server, "", a, b, &result) == SOAP_OK)
@@ -148,7 +150,7 @@ int main()
  *
 \******************************************************************************/
 
-#if defined(WITH_OPENSSL)
+#if defined(WITH_OPENSSL) && OPENSSL_VERSION_NUMBER < 0x10100000L
 
 struct CRYPTO_dynlock_value
 {
@@ -231,6 +233,8 @@ void CRYPTO_thread_cleanup()
 
 #else
 
+/* OpenSSL not used or OpenSSL prior to 1.1.0 */
+
 int CRYPTO_thread_setup()
 {
   return SOAP_OK;
